#pragma once

#include "NPMessageHandler.h"
#include "UpdaterDocument.h"
#include "MVC/Command/NotifyWindowCommand.h"
#include "CheckServiceUpgradeHelper.h"
#include "TimerEventTrigger.h"
#include <list>

namespace tipupdater
{

class CheckServiceUpgradeEventTrigger : public TimerEventTrigger
{
public:
	CheckServiceUpgradeEventTrigger()
	{
		m_next_show_interval = 60 * 60 * 24; 
	}

	void SetCheckTimeInterval(int interval)
	{
		m_next_show_interval = interval;
	}

	void SetDocument( UpdaterDocument *doc )
	{
		m_doc = doc;
	}

	void Error()
	{

	}

	BOOL Run()
	{
		CheckUpgradeService();
		return TRUE;
	}
private:
	void SendCreateNotifyUpgradeInformationCommand(TipServiceData *service)
	{
		CreateNotifyUpgradeInformationCommand* command = dynamic_cast<CreateNotifyUpgradeInformationCommand*>(TipCommandFactory::get_instance()->CreateAction(_T("CreateNotifyUpgradeInformationCommand")));
		if( NULL == command )
		{
			return;
		}

		command->SetGuid( service->GetID() , service->GetInstallInfo() == OldVersion);
		command->Execute();
	}

	int GetCheckTimeInterval()
	{
		return m_next_show_interval;
	}

	void CheckUpgradeService()
	{
		OutputDebugString(_T("CheckUpgradeService Start"));
		if( m_doc->GetUpdaterConfigModel()->IsUseNotifyUpdate() == FALSE )
		{
			OutputDebugString(_T("CheckUpgradeService Option is off"));
			return;
		}
		if( m_doc->IsUpdating() )
		{
			OutputDebugString(_T("CheckUpgradeService Now Updating"));
			return;
		}

		int count = m_doc->GetServiceCount();
		TipServiceData *service = NULL;
		AgentDLLWrapper* agentDllWrapper = NULL;
		NAVERAPP_INSTALL_STATUS InstallStatus;

		for (int i = 0; i < count; i++)
		{
			service = m_doc->GetService(i);

			CheckServiceUpgradeHelper helper(service);
			if( FALSE == helper.IsOverCheckTime() )
			{
				CString data;
				data.Format(_T("CheckUpgradeService Service Timeout %d"), i);
				OutputDebugString(data);
				continue;
			}
			
			helper.SetCheckTimeFromCurrentTime( GetCheckTimeInterval() );
			agentDllWrapper = service->GetAgentDll();
			if (agentDllWrapper)
			{
				agentDllWrapper->GetInstalledStatus(InstallStatus);
				switch(InstallStatus)
				{
				case NotInstalled:
				case OldVersion:
					SendCreateNotifyUpgradeInformationCommand(service);
					break;
				default:
					break;
				}
				agentDllWrapper = NULL;
			}
		}

		m_doc->SaveCheckEventTime();
	}

private:
	UpdaterDocument*			m_doc;
	int							m_next_show_interval;
};

};